import os
from stripe import StripeClient
from stripe.events import V1BillingMeterErrorReportTriggeredEvent

from flask import Flask, request, jsonify

app = Flask(__name__)
api_key = os.environ.get("STRIPE_API_KEY")
webhook_secret = os.environ.get("WEBHOOK_SECRET")

client = StripeClient(api_key)


@app.route("/webhook", methods=["POST"])
def webhook():
    webhook_body = request.data
    sig_header = request.headers.get("Stripe-Signature")

    try:
        thin_event = client.parse_thin_event(
            webhook_body, sig_header, webhook_secret
        )

        # Fetch the event data to understand the failure
        event = client.v2.core.events.retrieve(thin_event.id)
        if isinstance(event, V1BillingMeterErrorReportTriggeredEvent):
            meter = event.fetch_related_object()
            meter_id = meter.id
            print("Success! " + str(meter_id))

            # Record the failures and alert your team
            # Add your logic here

        return jsonify(success=True), 200
    except Exception as e:
        return jsonify(error=str(e)), 400


if __name__ == "__main__":
    app.run(port=4242)



import json
from typing import Callable

import pytest

import stripe
from stripe import ThinEvent
from tests.test_webhook import DUMMY_WEBHOOK_SECRET, generate_header

EventParser = Callable[[str], ThinEvent]


class TestV2Event(object):
    @pytest.fixture(scope="function")
    def v2_payload_no_data(self):
        return json.dumps(
            {
                "id": "evt_234",
                "object": "v2.core.event",
                "type": "financial_account.balance.opened",
                "livemode": True,
                "created": "2022-02-15T00:27:45.330Z",
                "related_object": {
                    "id": "fa_123",
                    "type": "financial_account",
                    "url": "/v2/financial_accounts/fa_123",
                    "stripe_context": "acct_123",
                },
                "reason": {
                    "id": "foo",
                    "idempotency_key": "bar",
                },
            }
        )

    @pytest.fixture(scope="function")
    def v2_payload_with_data(self):
        return json.dumps(
            {
                "id": "evt_234",
                "object": "v2.core.event",
                "type": "financial_account.balance.opened",
                "livemode": False,
                "created": "2022-02-15T00:27:45.330Z",
                "related_object": {
                    "id": "fa_123",
                    "type": "financial_account",
                    "url": "/v2/financial_accounts/fa_123",
                    "stripe_context": "acct_123",
                },
                "data": {
                    "containing_compartment_id": "compid",
                    "id": "foo",
                    "type": "bufo",
                },
            }
        )

    @pytest.fixture(scope="function")
    def stripe_client(self, http_client_mock):
        return stripe.StripeClient(
            api_key="keyinfo_test_123",
            stripe_context="wksp_123",
            http_client=http_client_mock.get_mock_http_client(),
        )

    @pytest.fixture(scope="function")
    def parse_thin_event(
        self, stripe_client: stripe.StripeClient
    ) -> EventParser:
        """
        helper to simplify parsing and validating events given a payload
        returns a function that has the client pre-bound
        """

        def _parse_thin_event(payload: str):
            return stripe_client.parse_thin_event(
                payload, generate_header(payload=payload), DUMMY_WEBHOOK_SECRET
            )

        return _parse_thin_event

    def test_parses_thin_event(
        self, parse_thin_event: EventParser, v2_payload_no_data: str
    ):
        event = parse_thin_event(v2_payload_no_data)

        assert isinstance(event, ThinEvent)
        assert event.id == "evt_234"

        assert event.related_object
        assert event.related_object.id == "fa_123"

        assert event.reason
        assert event.reason.id == "foo"

    def test_parses_thin_event_with_data(
        self, parse_thin_event: EventParser, v2_payload_with_data: str
    ):
        event = parse_thin_event(v2_payload_with_data)

        assert isinstance(event, ThinEvent)
        assert not hasattr(event, "data")
        assert event.reason is None

    def test_validates_signature(
        self, stripe_client: stripe.StripeClient, v2_payload_no_data
    ):
        with pytest.raises(stripe.error.SignatureVerificationError):
            stripe_client.parse_thin_event(
                v2_payload_no_data, "bad header", DUMMY_WEBHOOK_SECRET
            )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._product_feature import ProductFeature
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import List, cast
from typing_extensions import NotRequired, TypedDict


class ProductFeatureService(StripeService):
    class CreateParams(TypedDict):
        entitlement_feature: str
        """
        The ID of the [Feature](https://stripe.com/docs/api/entitlements/feature) object attached to this product.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    class DeleteParams(TypedDict):
        pass

    class ListParams(TypedDict):
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    def delete(
        self,
        product: str,
        id: str,
        params: "ProductFeatureService.DeleteParams" = {},
        options: RequestOptions = {},
    ) -> ProductFeature:
        """
        Deletes the feature attachment to a product
        """
        return cast(
            ProductFeature,
            self._request(
                "delete",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        product: str,
        id: str,
        params: "ProductFeatureService.DeleteParams" = {},
        options: RequestOptions = {},
    ) -> ProductFeature:
        """
        Deletes the feature attachment to a product
        """
        return cast(
            ProductFeature,
            await self._request_async(
                "delete",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        product: str,
        id: str,
        params: "ProductFeatureService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> ProductFeature:
        """
        Retrieves a product_feature, which represents a feature attachment to a product
        """
        return cast(
            ProductFeature,
            self._request(
                "get",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        product: str,
        id: str,
        params: "ProductFeatureService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> ProductFeature:
        """
        Retrieves a product_feature, which represents a feature attachment to a product
        """
        return cast(
            ProductFeature,
            await self._request_async(
                "get",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        product: str,
        params: "ProductFeatureService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[ProductFeature]:
        """
        Retrieve a list of features for a product
        """
        return cast(
            ListObject[ProductFeature],
            self._request(
                "get",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        product: str,
        params: "ProductFeatureService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[ProductFeature]:
        """
        Retrieve a list of features for a product
        """
        return cast(
            ListObject[ProductFeature],
            await self._request_async(
                "get",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        product: str,
        params: "ProductFeatureService.CreateParams",
        options: RequestOptions = {},
    ) -> ProductFeature:
        """
        Creates a product_feature, which represents a feature attachment to a product
        """
        return cast(
            ProductFeature,
            self._request(
                "post",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        product: str,
        params: "ProductFeatureService.CreateParams",
        options: RequestOptions = {},
    ) -> ProductFeature:
        """
        Creates a product_feature, which represents a feature attachment to a product
        """
        return cast(
            ProductFeature,
            await self._request_async(
                "post",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
